@@ -8,23 +8,27 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.0.14'
+__version__ = '0.0.15'
+
+import sys
 
 from wandb.errors import Error
 
-import sys
-if sys.version_info < (3, 0):
-    from wandb.sdk_py27.wandb_init import init
-    from wandb.sdk_py27.wandb_setup import setup
-    from wandb.sdk_py27.wandb_save import save
-    from wandb.sdk_py27.wandb_watch import watch
-    from wandb.sdk_py27.wandb_login import login
+
+PY3 = sys.version_info.major == 3 and sys.version_info.minor >= 6
+if PY3:
+    TYPE_CHECKING = True
+    from wandb import sdk as wandb_sdk
 else:
-    from wandb.sdk.wandb_init import init
-    from wandb.sdk.wandb_setup import setup
-    from wandb.sdk.wandb_save import save
-    from wandb.sdk.wandb_watch import watch
-    from wandb.sdk.wandb_login import login
+    TYPE_CHECKING = False
+    from wandb import sdk_py27 as wandb_sdk
+
+init = wandb_sdk.init
+setup = wandb_sdk.setup
+save = wandb_sdk.save
+watch = wandb_sdk.watch
+login = wandb_sdk.login
+Settings = wandb_sdk.Settings
 
 from wandb.util import preinit as _preinit
 from wandb.errors.term import termlog, termerror, termwarn
