@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Compatibility keras module.
 
@@ -6,6 +5,6 @@ In the future use:
     from wandb.framework.keras import WandbCallback
 """
 
-from wandb.framework.keras import WandbCallback
+from wandb.framework.keras import WandbCallback  # type: ignore
 
 __all__ = ['WandbCallback']
