@@ -10,11 +10,11 @@ from .wandb_run import Run
 from wandb.util.globals import set_global
 from wandb.backend.backend import Backend
 from wandb.stuff import util2
+from wandb.util import reporting
 
 import time
 import json
 import atexit
-import platform
 import six
 import logging
 from six import raise_from
@@ -23,36 +23,13 @@ import sys
 import os
 from wandb.util import redirect
 
-# from wandb.apis import internal
+from .wandb_settings import Settings
 
-# import typing
-# if typing.TYPE_CHECKING:
-#   from typing import Dict, List, Optional
-# from typing import Optional, Dict
-from typing import Optional, Union, List, Dict  # noqa: F401
+if wandb.TYPE_CHECKING:
+    from typing import Optional, Union, List, Dict, Any  # noqa: F401
 
 logger = logging.getLogger("wandb")
 
-# priority order (highest to lowest):
-# WANDB_FORCE_MODE
-# settings.force_mode
-# wandb.init(mode=)
-# WANDB_MODE
-# settings.mode
-# ) -> Optional[Run]:
-
-# def init(settings: Dict = None,
-#          mode: int = None,
-#          entity=None,
-#          team=None,
-#          project=None,
-#          group=None,
-#          magic=None,
-#          config=None,
-#          reinit=None,
-#          name=None,
-#          ) -> Optional[Run]:
-
 
 def online_status(*args, **kwargs):
     pass
@@ -107,6 +84,7 @@ class _WandbInit(object):
         self._redirect_cb = None
         self._out_redir = None
         self._err_redir = None
+        self._reporter = None
 
         # move this
         self.stdout_redirector = None
@@ -114,10 +92,16 @@ class _WandbInit(object):
         self._save_stdout = None
         self._save_stderr = None
 
+        self._atexit_cleanup_called = None
+
     def setup(self, kwargs):
         self.kwargs = kwargs
 
-        settings = kwargs.pop("settings", None)
+        wl = wandb.setup()
+        settings: Settings = wl.settings(dict(kwargs.pop("settings", tuple())))
+
+        self._reporter = reporting.setup_reporter(
+            settings=settings.duplicate().freeze())
 
         # Remove parameters that are not part of settings
         self.config = kwargs.pop("config", None)
@@ -140,24 +124,21 @@ class _WandbInit(object):
         for key in unsupported:
             val = kwargs.pop(key, None)
             if val:
-                logger.info("unsupported wandb.init() arg: %s", val)
+                self._reporter.warning("unsupported wandb.init() arg: %s", key)
 
-        wl = wandb.setup()
-        settings = settings or dict()
-        s = wl.settings(**settings)
-        d = dict(**kwargs)
-        # strip out items where value is None
-        d = {k: v for k, v in six.iteritems(d) if v is not None}
+        settings.apply_init(kwargs)
 
         # TODO(jhr): should this be moved? probably.
-        d.setdefault("start_time", time.time())
+        d = dict(start_time=time.time())
+        settings.update(d)
 
-        s.update(d)
-        s.freeze()
         self.wl = wl
-        self.settings = s
+        self.settings = settings.freeze()
 
     def _atexit_cleanup(self):
+        if self._atexit_cleanup_called:
+            return
+        self._atexit_cleanup_called = True
 
         ret = self.backend.interface.send_exit_sync(0, timeout=30)
         logger.info("got exit ret: %s", ret)
@@ -174,36 +155,30 @@ class _WandbInit(object):
         self.backend.interface.send_output(name, data)
 
     def _redirect(self, stdout_slave_fd, stderr_slave_fd):
-        logger.info("redirect")
+        console = self.settings.console
+        logger.info("redirect: %s", console)
 
-        if self._use_redirect:
-            out = False
-            err = False
-            out = True
-            err = True
-            if out:
-                out_cap = redirect.Capture(name="stdout", cb=self._redirect_cb)
-                out_redir = redirect.Redirect(src="stdout",
-                                              dest=out_cap,
-                                              unbuffered=True,
-                                              tee=True)
-            if err:
-                err_cap = redirect.Capture(name="stderr", cb=self._redirect_cb)
-                err_redir = redirect.Redirect(src="stderr",
-                                              dest=err_cap,
-                                              unbuffered=True,
-                                              tee=True)
-            if out:
-                out_redir.install()
-            if err:
-                err_redir.install()
-            if out:
-                self._out_redir = out_redir
-            if err:
-                self._err_redir = err_redir
+        if console == 'redirect':
+            logger.info("redirect1")
+            out_cap = redirect.Capture(name="stdout", cb=self._redirect_cb)
+            out_redir = redirect.Redirect(src="stdout",
+                                          dest=out_cap,
+                                          unbuffered=True,
+                                          tee=True)
+            err_cap = redirect.Capture(name="stderr", cb=self._redirect_cb)
+            err_redir = redirect.Redirect(src="stderr",
+                                          dest=err_cap,
+                                          unbuffered=True,
+                                          tee=True)
+            out_redir.install()
+            err_redir.install()
+            self._out_redir = out_redir
+            self._err_redir = err_redir
             logger.info("redirect2")
             return
 
+        return
+
         # redirect stdout
         if platform.system() == "Windows":
             win32_redirect(stdout_slave_fd, stderr_slave_fd)
@@ -257,27 +232,35 @@ class _WandbInit(object):
                 config[k] = v
 
         if s.mode == "noop":
+            # TODO(jhr): return dummy object
             return None
 
         # Make sure we are logged in
         wandb.login()
 
-        if self._use_redirect:
-            stdout_master_fd = None
-            stderr_master_fd = None
-            stdout_slave_fd = None
-            stderr_slave_fd = None
-            #self._redirect_q = self.wl._multiprocessing.Queue()
+        stdout_master_fd = None
+        stderr_master_fd = None
+        stdout_slave_fd = None
+        stderr_slave_fd = None
+        console = s.console
+
+        if console == 'redirect':
+            pass
+        elif console == 'off':
+            pass
+        elif console == 'mock':
+            pass
+        elif console == 'file':
+            pass
+        elif console == 'iowrap':
+            stdout_master_fd, stdout_slave_fd = io_wrap.wandb_pty(resize=False)
+            stderr_master_fd, stderr_slave_fd = io_wrap.wandb_pty(resize=False)
+        elif console == '_win32':
+            # Not used right now
+            stdout_master_fd, stdout_slave_fd = win32_create_pipe()
+            stderr_master_fd, stderr_slave_fd = win32_create_pipe()
         else:
-            if platform.system() == "Windows":
-                # create win32 pipes
-                stdout_master_fd, stdout_slave_fd = win32_create_pipe()
-                stderr_master_fd, stderr_slave_fd = win32_create_pipe()
-            else:
-                stdout_master_fd, stdout_slave_fd = io_wrap.wandb_pty(
-                    resize=False)
-                stderr_master_fd, stderr_slave_fd = io_wrap.wandb_pty(
-                    resize=False)
+            self._reporter.internal("Unknown console: %s", console)
 
         backend = Backend(mode=s.mode)
         backend.ensure_launched(
@@ -294,6 +277,7 @@ class _WandbInit(object):
 
         run = Run(config=config, settings=s)
         run._set_backend(backend)
+        run._set_reporter(self._reporter)
         # TODO: pass mode to backend
         run_synced = None
 
@@ -319,7 +303,9 @@ class _WandbInit(object):
         self.run = run
         self.backend = backend
         set_global(run=run, config=run.config, log=run.log, join=run.join)
+        self._reporter.set_context(run=run)
         run.on_start()
+
         logger.info("atexit reg")
         atexit.register(lambda: self._atexit_cleanup())
 
@@ -341,10 +327,10 @@ def getcaller():
 
 
 def init(
+        settings: Union[Settings, Dict[str, Any], str, None] = None,
         entity: Optional[str] = None,
         team: Optional[str] = None,
         project: Optional[str] = None,
-        settings: Optional[Dict] = None,
         mode: Optional[str] = None,
         group: Optional[str] = None,
         job_type: Optional[str] = None,
@@ -366,7 +352,7 @@ def init(
         sync_tensorboard=None,
         monitor_gym=None,
         id=None,
-) -> Optional[Run]:
+) -> Run:
     """Initialize a wandb Run.
 
     Args:
@@ -390,12 +376,11 @@ def init(
         except (KeyboardInterrupt, Exception) as e:
             getcaller()
             logger.exception("we got issues")
+            wi._atexit_cleanup()
             if wi.settings.problem == "fatal":
                 raise
             if wi.settings.problem == "warn":
                 pass
-            # silent or warn
-            # TODO: return dummy run instead
             return None
     except KeyboardInterrupt as e:
         logger.warning("interupted", exc_info=e)
